import 'dart:convert';
import 'dart:io';
import 'package:dio/dio.dart';
import '../Alert/Ev_Alert.dart';
import '../Alert/Ev_Loading.dart';
import '../System/Ev_Global.dart';
import '../System/Ev_Lang.dart';
import 'package:flutter/cupertino.dart';

class Ev_API_Result
{  var result;
   var sdata;
   var data;
}

class Ev_API
{
    CancelToken token;
    Dio dio = Dio();
    String Api_url;
    var PostData;

    Ev_API() {
      dio.options.connectTimeout = 5000;
      dio.options.receiveTimeout = 15000;
    }

    void Popup_Post(BuildContext context,String url,{Map<String,String> post,Map<String,File> uploadFile,onProgress,onSuccess})
    {    Post(url,context:context,
            post:post,
            uploadFile:uploadFile,
            onProgress:onProgress,
            onSuccess:onSuccess,
            onConnectError:()
            {  Ev_Alert.Error(context, api_nonetwork[Ev_Lang.lang_id]); },
            alert_error:true,
            pop_loading:true);
    }

    Future<Ev_API_Result> Post(String url,{BuildContext context,Map<String,String> post,Map<String,File> uploadFile,onProgress,onSuccess,onConnectError,onReturnError,alert_error = false,pop_loading = false})
    async {
      if(pop_loading && context==null)
        {  print("API Error on Pop Loading is True (BuildContext is Null)");
           exit(0);
        }

      print("Post API " + post.toString());
      token = CancelToken();

      Map<String, dynamic> fordata = Map();
      fordata["lang"] = Ev_Lang.lang_id;

      if (post != null) {
        post.forEach((String key, String value) {
          fordata[key] = value;
        });
      }

      if(uploadFile != null)
       { uploadFile.forEach((String key, File file) {
          fordata[key] = new UploadFileInfo(file, file.path);
        });}

      PostData = new FormData.from(fordata);
      Api_url = url;

      print("Post API $Api_url :" + fordata.toString());

      if(pop_loading)
         { Ev_Loading.show(context); }

      Response response = await dio.post(url, data: PostData, cancelToken: token,

       onSendProgress: (int sent, int total) {
          if(onProgress != null && uploadFile != null)
           {  onProgress(sent, total); }},)

      .catchError((err) {
        token = null;
        Ev_Loading.hide(context);

        if(err.type == DioErrorType.CANCEL) {
          print("API Cancel");
          return;
        }

        print("Catch Error " + err.message.toString() + "." + err.type.toString());

        if (err.type == DioErrorType.RESPONSE) {
          print(url + " Response ");
        }
        if (err.type == DioErrorType.CONNECT_TIMEOUT) {
          print(url + "Connect Timeout ");
        }
        if (err.type == DioErrorType.RECEIVE_TIMEOUT) {
          print(url + "Receive Timeout ");
        }

        if (onConnectError != null) {
          onConnectError();
        }
        return ;
      }); // end CatchError

      token = null;
      Ev_Loading.hide(context);

      if (response != null && response.statusCode == 200) {
        print("$url API Success Response :" + response.statusCode.toString() + " : " + response.toString());


        var api_return = Ev_API_Result();
        var decode;

        try{
            decode =  jsonDecode(response.toString());
            print(decode);
          } on FormatException catch(e){
          print("API Result Error!");
           if (onReturnError != null) {
             onReturnError();
           }
           return null;
        }

        api_return.result = decode;
        api_return.sdata = decode["sdata"];
        api_return.data = decode["data"];

        var js = api_return.result;
        if (js["status"] == 0 && js["err_message"]
            .toString()
            .isNotEmpty && alert_error) {
            Ev_Alert.Error(context, js["err_message"].toString());

            if(onReturnError!=null)
              {  onReturnError(); }
        }

        if (js["status"] == 1 && onSuccess != null) {
          onSuccess(api_return);
        }

        return api_return;
      }

      return null;
    }

    void Stop() {
      if( token != null) {
        token.cancel();
        token = null;
      }
    }
}